#!/usr/bin/env ruby -s

require 'erb'

def markdown_text (text)
  text.gsub(/`(.*?)`/, '<code>\1</code>')
end

def make_id (text)
  text = text.gsub(/`(.*?)`/, '\1')
  text.gsub!(/[- \/_]+/, '_')
  text.gsub!(/[^a-zA-Z0-9_]+/, '')
  text.downcase
end

$toc = [ "<h1>Table of Contents</h1>" ]
$chapters = [ ]

$toc_template     = "templates/toc.rhtml"       if $toc_template.nil?
$chapter_template = "templates/chapter.rhtml"   if $chapter_template.nil?
$all_template     = "templates/all.rhtml"       if $all_template.nil?
$output_dir       = "html"                      if $output_dir.nil?

files = Dir.entries("pages").sort.grep(/.*\.markdown/)
files.each_with_index do |file, i|

  def html_link (markdown_file)
    markdown_file.sub(/^(?:\d+_)?(.*?)\.markdown$/, '\1.html')
  end

  puts "• Convert #{file}…"
  $toc.push("<ol>")

  IO.popen("Markdown.pl|{ cd images; ../add_image_size.rb; }", "r+") do |f|

    Thread.fork do
      heading_stack = [ i ]
      File.open("pages/#{file}").each_line do |line|

        if(m = line.match(/^(#+)\s+(.*)$/))
          level = m[1].length
          title = m[2]

          if heading_stack.length < level
            heading_stack.push(0)
            $toc.push("<ol>")
          else
            while heading_stack.length > level
              heading_stack.pop
              $toc.push("</ol>")
            end
          end

          heading_stack.push(heading_stack.pop + 1)
          prefix = heading_stack.join(".")
          puts("   #{prefix} #{title}")

          $toc.push("<li>#{"&#x2007;" if(heading_stack.first < 10)}#{prefix}&#x2003;<a href='#{html_link file}##{make_id title}'>#{markdown_text title}</a></li>")
          f.write("#{m[1]} <span id='#{make_id title}'>#{prefix}</span> #{title}")
        else
          f.write(line)
        end

      end

      f.write(File.open("markdown.references").read)
      f.close_write

      heading_stack.length.times { $toc.push("</ol>") }
    end

    $page = { :title => %x{ grep '^#' 'pages/#{file}'|head -n1 }.sub(/^#+\s+(.*)\n/, '\1'), :content => f.read, :name => "chapter" }
    $prev = i > 0 ? { :link => "#{html_link files[i-1]}", :title => markdown_text(%x{ grep '^#' 'pages/#{files[i-1]}'|head -n1 }.sub(/^#+\s+(.*)\n/, '\1')) } : nil
    $next = i + 1 < files.length  ? { :link => "#{html_link files[i+1]}", :title => markdown_text(%x{ grep '^#' 'pages/#{files[i+1]}'|head -n1 }.sub(/^#+\s+(.*)\n/, '\1')) } : nil

    File.open("#{$output_dir}/#{html_link file}", "w") do |f|
      f.write(ERB.new(File.open($chapter_template), 0, '<>').result)
    end

    $chapters.push($page)

  end

  File.open("#{$output_dir}/all_pages.html", "w") do |f|
    f.write(ERB.new(File.open($all_template), 0, '<>').result)
  end

end

File.open("#{$output_dir}/index.html", "w") do |f|
  $page = { :content => $toc.join("\n"), :name => "toc", :title => "Table of Contents" }
  f.write(ERB.new(File.open($toc_template), 0, '<>').result)
end
